package com.example.api.core

import com.example.api.core.data.core.ApiService
import com.example.api.core.data.dto.RegisterUserDto
import com.example.api.core.data.dto.SignInUserDto
import com.jakewharton.retrofit2.converter.kotlinx.serialization.asConverterFactory
import junit.framework.TestCase.assertEquals
import kotlinx.coroutines.runBlocking
import kotlinx.serialization.json.Json
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import okhttp3.mockwebserver.MockResponse
import okhttp3.mockwebserver.MockWebServer
import org.junit.After
import org.junit.Before
import org.junit.Test
import retrofit2.Retrofit
import retrofit2.create

/**
 * Автор: Манякин Дмитрий (user5)
 * Дата создания: 27.05.2025
 * */

class ServiceTest {

    private lateinit var mockWebServer: MockWebServer
    private lateinit var service: ApiService

    @Before
    fun setup() {
        mockWebServer = MockWebServer()
        mockWebServer.start()

        val json = Json { ignoreUnknownKeys = true }

        service = Retrofit.Builder()
            .baseUrl(mockWebServer.url("/"))
            .addConverterFactory(json.asConverterFactory("application/json".toMediaType()))
            .client(OkHttpClient.Builder().addInterceptor(HttpLoggingInterceptor().apply {
                level = HttpLoggingInterceptor.Level.BODY
            }).build())
            .build().create<ApiService>()
    }

    @Test
    fun test_registration() = runBlocking {
        val response = """
            {
              "collectionId": "_pb_users_auth_",
              "collectionName": "users",
              "created": "2025-05-27 06:08:13.833Z",
              "dateBirthday": "",
              "emailVisibility": false,
              "firstname": "",
              "gender": "",
              "id": "w20q0pj1e2824oz",
              "lastname": "",
              "secondname": "",
              "updated": "2025-05-27 06:08:13.833Z",
              "verified": false
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.register(
            RegisterUserDto(
                email = "email@test.com",
                password = "12345678",
                passwordConfirm = "12345678"
            )
        )

        val request = mockWebServer.takeRequest()

        assertEquals("w20q0pj1e2824oz", responseResult.id)
        assertEquals("2025-05-27 06:08:13.833Z", responseResult.created)

        assertEquals("POST", request.method)
        assertEquals("/collections/users/records", request.path)
    }

    @Test
    fun test_sign_in() = runBlocking {
        val response = """
            {
              "record": {
                "collectionId": "_pb_users_auth_",
                "collectionName": "users",
                "created": "2025-05-27 06:08:13.833Z",
                "dateBirthday": "",
                "email": "example123@test.ru",
                "emailVisibility": false,
                "firstname": "",
                "gender": "",
                "id": "w20q0pj1e2824oz",
                "lastname": "",
                "secondname": "",
                "updated": "2025-05-27 06:08:13.833Z",
                "verified": false
              },
              "token": "eyJhbGciOiJIUzIqwwe1NiIsInR5cCI6IkpXVCJg9.eyJjb2xsZWN0aW9uSWheQiOiJfqcGggJfdXNlcnNfYXV0aFasd8iLCJleHAiOjE3NDg5MzA5MjAsImlkIjoidzIwcTBwajFlMjgyNG96IiwicmVmcmVzaGFibGUiOnRydWUsInR5cGUiOiJhdXRoIn0.7Sc_NB_CgnwEfXWp0NjsnDT35gGVXzFJwwLUVD730Gw"
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.auth(
            SignInUserDto(
                identity = "example123@test.ru",
                password = "12345678"
            )
        )

        val request = mockWebServer.takeRequest()

        assertEquals("example123@test.ru", responseResult.record.email)
        assertEquals(
            "eyJhbGciOiJIUzIqwwe1NiIsInR5cCI6IkpXVCJg9.eyJjb2xsZWN0aW9uSWheQiOiJfqcGggJfdXNlcnNfYXV0aFasd8iLCJleHAiOjE3NDg5MzA5MjAsImlkIjoidzIwcTBwajFlMjgyNG96IiwicmVmcmVzaGFibGUiOnRydWUsInR5cGUiOiJhdXRoIn0.7Sc_NB_CgnwEfXWp0NjsnDT35gGVXzFJwwLUVD730Gw",
            responseResult.token
        )

        assertEquals("POST", request.method)
        assertEquals("/collections/users/auth-with-password", request.path)
    }

    @Test
    fun test_project_list() = runBlocking {
        val response = """
            {
              "items": [
                {
                  "category": "е",
                  "collectionId": "pbc_3202395908",
                  "collectionName": "project",
                  "created": "2025-05-26 14:47:25.523Z",
                  "dateEnd": "2025-05-24 12:00:00.000Z",
                  "dateStart": "2025-05-15 12:00:00.000Z",
                  "description_source": "е",
                  "gender": "е",
                  "id": "7m35pcmbjr86kh4",
                  "image": "",
                  "title": "е",
                  "typeProject": "е",
                  "updated": "2025-05-26 14:47:25.523Z",
                  "user_id": ""
                }
              ],
              "page": 1,
              "perPage": 30,
              "totalItems": 1,
              "totalPages": 1
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.projects()

        val request = mockWebServer.takeRequest()

        assertEquals(1, responseResult.page)
        assertEquals(1, responseResult.totalItems)
        assertEquals(1, responseResult.items.size)

        assertEquals("GET", request.method)
        assertEquals("/collections/project/records", request.path)
    }

    @Test
    fun test_sales_and_news() = runBlocking {
        val response = """
            {
              "items": [
                {
                  "collectionId": "pbc_2599178718",
                  "collectionName": "News",
                  "created": "2025-05-26 14:21:46.284Z",
                  "id": "yxe08bp4woz996w",
                  "newsImage": "banner_u9uqsy4sf9.png",
                  "updated": "2025-05-26 15:29:41.083Z"
                },
                {
                  "collectionId": "pbc_2599178718",
                  "collectionName": "News",
                  "created": "2025-05-26 15:29:50.831Z",
                  "id": "lqg3hpgn46efgmz",
                  "newsImage": "banner_2_10pjar2fq7.png",
                  "updated": "2025-05-26 15:29:50.831Z"
                }
              ],
              "page": 1,
              "perPage": 30,
              "totalItems": 2,
              "totalPages": 1
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.salesAndNews()

        val request = mockWebServer.takeRequest()

        assertEquals(2, responseResult.items.size)
        assertEquals("News", responseResult.items.first().collectionName)
        assertEquals("lqg3hpgn46efgmz", responseResult.items[1].id)

        assertEquals("GET", request.method)
        assertEquals("/collections/news/records", request.path)
    }

    @After
    fun teardown() {
        mockWebServer.shutdown()
    }
}